package com.android.sample;

import android.app.Dialog;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import com.android.sample.enums.ApiInterface;
import com.android.sample.models.PurchaseData;
import com.android.sample.usdk.InitializeUsdkResultCallback;
import com.android.sample.usdk.PureInterfaceUsdkManager;
import com.android.sample.usdk.PureInterfaceWithScaUsdkManager;
import com.android.sample.usdk.SimplifiedInterfaceUsdkManager;
import com.android.sample.usdk.UsdkManager;
import com.usdk.android.UsdkThreeDS2ServiceImpl;

import java.util.ArrayList;
import java.util.List;

import static android.Manifest.permission.ACCESS_FINE_LOCATION;
import static android.Manifest.permission.READ_PHONE_STATE;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = MainActivity.class.getSimpleName();

    private static final String[] REQUIRED_PERMISSIONS = new String[] {ACCESS_FINE_LOCATION, READ_PHONE_STATE};
    private static final int PERMISSIONS_REQUEST_CODE = 1;

    private static UsdkManager usdkManager;

    private Settings settings;

    private EditText emailEdit;
    private EditText passwordEdit;
    private EditText accountNumberEdit;
    private EditText backendUrlEdit;
    private EditText licenseKeyEdit;
    private EditText sentryDsnEdit;
    private EditText focusedEditText;
    private RadioGroup selectInterfaceGroup;
    private Button simulateCrushBtn;
    private Button purchaseBtn;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            Log.d(TAG, "Request permissions.");
            requestPermissions();
        }

        settings = new Settings(this);

        TextView buildInfoTxt = findViewById(R.id.build_info_txt);

        buildInfoTxt.setText(String.format("app: %s, usdk: %s", BuildConfig.APP_VERSION, new UsdkThreeDS2ServiceImpl().getSDKVersion()));

        emailEdit = findViewById(R.id.email_edit);
        emailEdit.setText(settings.getEmail());
        emailEdit.setOnFocusChangeListener(onFocusChangeListener);

        passwordEdit = findViewById(R.id.password_edit);
        passwordEdit.setText(settings.getPassword());
        passwordEdit.setOnFocusChangeListener(onFocusChangeListener);

        licenseKeyEdit = findViewById(R.id.license_key_edit);
        licenseKeyEdit.setText(settings.getLicenseKey());
        licenseKeyEdit.setOnFocusChangeListener(onFocusChangeListener);

        accountNumberEdit = findViewById(R.id.account_number_edit);
        accountNumberEdit.setText(settings.getAccountNumber());
        accountNumberEdit.setOnFocusChangeListener(onFocusChangeListener);

        backendUrlEdit = findViewById(R.id.backend_url_edit);
        backendUrlEdit.setText(settings.getBackendUrl());
        backendUrlEdit.setOnFocusChangeListener(onFocusChangeListener);

        sentryDsnEdit = findViewById(R.id.sentry_dsn_edit);
        sentryDsnEdit.setText(settings.getSentryDsn());
        sentryDsnEdit.setOnFocusChangeListener(onFocusChangeListener);

        simulateCrushBtn = findViewById(R.id.simulate_crush_btn);
        simulateCrushBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!TextUtils.isEmpty(settings.getSentryDsn())) {
                    throw new RuntimeException("Simulated crash");
                } else {
                    showDialog("Sentry.io DSN should be populated", "OK");
                }
            }
        });

        selectInterfaceGroup = findViewById(R.id.select_api_group);

        RadioButton checkedRadioButton;
        ApiInterface apiInterface = settings.getApiInterface();
        if (apiInterface == ApiInterface.PURE) {
            checkedRadioButton = findViewById(R.id.pure_interface_check_box);
        }
        else if (apiInterface == ApiInterface.PURE_WITH_SCA) {
            checkedRadioButton = findViewById(R.id.pure_interface_with_sca_check_box);
        }
        else {
            checkedRadioButton = findViewById(R.id.simplified_check_box);
        }
        checkedRadioButton.setChecked(true);

        purchaseBtn = findViewById(R.id.purchase_btn);

        purchaseBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                final String licenseKey = licenseKeyEdit.getText().toString();
                if (TextUtils.isEmpty(licenseKey)) {
                    showDialog("Please, enter license key", "OK");
                    return;
                }

                ApiInterface selectedApiInterface;
                if (selectInterfaceGroup.getCheckedRadioButtonId() == R.id.simplified_check_box) {
                    selectedApiInterface = ApiInterface.SIMPLIFIED;
                }
                else if (selectInterfaceGroup.getCheckedRadioButtonId() == R.id.pure_interface_check_box) {
                    selectedApiInterface = ApiInterface.PURE;
                }
                else {
                    selectedApiInterface = ApiInterface.PURE_WITH_SCA;
                }

                if (usdkManager == null
                        || !licenseKey.equals(settings.getLicenseKey())
                        || selectedApiInterface != settings.getApiInterface()) {

                    settings.saveApiInterface(selectedApiInterface);
                    if (selectedApiInterface == ApiInterface.SIMPLIFIED) {
                        usdkManager = new SimplifiedInterfaceUsdkManager(MainActivity.this, licenseKey);
                    } else if (selectedApiInterface == ApiInterface.PURE) {
                        usdkManager = new PureInterfaceUsdkManager(MainActivity.this, licenseKey);
                    } else if (selectedApiInterface == ApiInterface.PURE_WITH_SCA) {
                        usdkManager = new PureInterfaceWithScaUsdkManager(MainActivity.this, licenseKey);
                    }

                    usdkManager.initializeUsdk(new InitializeUsdkResultCallback() {
                        @Override
                        public void onSuccess() {
                            purchase();
                        }

                        @Override
                        public void onError(String message) {
                            showDialog(message, "OK");
                        }
                    });
                } else {
                    purchase();
                }
            }
        });
    }

    @Override
    protected void onStop() {
        onFocusChangeListener.onFocusChange(focusedEditText, false);

        super.onStop();
    }

    private void purchase() {
        final String email = emailEdit.getText().toString();
        if (TextUtils.isEmpty(email)) {
            showDialog("Please, enter email", "OK");
            return;
        }

        final String password = passwordEdit.getText().toString();
        if (TextUtils.isEmpty(password)) {
            showDialog("Please, enter password", "OK");
            return;
        }

        String cardNumber = accountNumberEdit.getText().toString();
        if (TextUtils.isEmpty(cardNumber)) {
            showDialog("Please, enter card number", "OK");
            return;
        }

        String backendUrl = backendUrlEdit.getText().toString();
        if (TextUtils.isEmpty(backendUrl)) {
            showDialog("Please, enter backend URL", "OK");
            return;
        }

        PurchaseData purchaseData = new PurchaseData();
        purchaseData.setAccountNumber(cardNumber);
        purchaseData.setEmail(email);
        purchaseData.setPassword(password);
        purchaseData.setBackendUrl(backendUrl);

        usdkManager.purchase(purchaseData);
    }

    private boolean requestPermissions() {
        List<String> permissionsForRequest = new ArrayList<>();

        for (String permission : REQUIRED_PERMISSIONS) {
            if (ContextCompat.checkSelfPermission(MainActivity.this, permission) != PackageManager.PERMISSION_GRANTED) {
                permissionsForRequest.add(permission);
            }
        }

        if (!permissionsForRequest.isEmpty()) {
            ActivityCompat.requestPermissions(MainActivity.this,
                    permissionsForRequest.toArray(new String[permissionsForRequest.size()]), PERMISSIONS_REQUEST_CODE);
        }

        return !permissionsForRequest.isEmpty();
    }

    public void showDialog(final String message, final String buttonText) {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                AlertDialog.Builder adb = new AlertDialog.Builder(MainActivity.this)
                        .setMessage(message);
                adb.setPositiveButton(buttonText, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {

                    }
                });

                Dialog dialog = adb.create();
                dialog.setCanceledOnTouchOutside(false);
                dialog.setCancelable(false);
                dialog.show();
            }
        });
    }

    private View.OnFocusChangeListener onFocusChangeListener = new View.OnFocusChangeListener() {
        @Override
        public void onFocusChange(View view, boolean hasFocus) {
            if (!hasFocus) {
                if (view == emailEdit) {
                    settings.saveEmail(emailEdit.getText().toString());
                }
                else if (view == passwordEdit) {
                    settings.savePassword(passwordEdit.getText().toString());
                }
                else if (view == backendUrlEdit) {
                    settings.saveBackendUrl(backendUrlEdit.getText().toString());
                }
                else if (view == licenseKeyEdit) {
                    settings.saveLicenseKey(licenseKeyEdit.getText().toString());
                }
                else if (view == accountNumberEdit) {
                    settings.saveAccountNumber(accountNumberEdit.getText().toString());
                }
                else if (view == sentryDsnEdit) {
                    settings.saveSentryDsn(sentryDsnEdit.getText().toString());
                }
            } else {
                focusedEditText = (EditText) view;
            }
        }
    };
}
