package com.android.sample.usdk;

import android.app.ProgressDialog;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.android.sample.MainActivity;
import com.android.sample.R;
import com.android.sample.models.DirectoryServerItem;
import com.android.sample.models.PurchaseData;
import com.android.sample.models.PurchaseRequest;
import com.android.sample.models.PurchaseResponse;
import com.android.sample.models.SupportedVersionsResponse;
import com.google.gson.Gson;

import org.emvco.threeds.core.AuthenticationRequestParameters;
import org.emvco.threeds.core.ChallengeParameters;
import org.emvco.threeds.core.ChallengeStatusReceiver;
import org.emvco.threeds.core.CloseTransactionSpec;
import org.emvco.threeds.core.CompletionEvent;
import org.emvco.threeds.core.CreateTransactionSpec;
import org.emvco.threeds.core.MessageExtension;
import org.emvco.threeds.core.ProtocolErrorEvent;
import org.emvco.threeds.core.RuntimeErrorEvent;
import org.emvco.threeds.core.Transaction;
import org.emvco.threeds.core.UsdkPromptCallback;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

import static com.android.sample.models.PurchaseResponse.CHALLENGE_TRANS_STATUS;

public class PureInterfaceWithScaUsdkManager extends UsdkManager {

    private static final String TAG = SimplifiedInterfaceUsdkManager.class.getSimpleName();

    static final MediaType JSON = MediaType.parse("application/json; charset=utf-8");
    private static final String APP_DEVICE_CHANEL_VALUE = "01";
    private static final String PAYMENT_MESSAGE_CATEGORY_VALUE = "01";

    private ProgressDialog progress;
    private final OkHttpClient client = new OkHttpClient();

    public PureInterfaceWithScaUsdkManager(MainActivity activity, String licenseKey) {
        super(activity, licenseKey);
    }

    public void purchase(final PurchaseData purchaseData) {
        Log.d(TAG, "Purchase using pure interface with SCA support");
        progress = new ProgressDialog(activity);
        progress.setMessage(activity.getString(R.string.progress_get_supported_versions_by_ds));
        progress.setIndeterminate(true);
        progress.setCancelable(false);
        progress.show();

        getSupportedVersionsByDsRequest(purchaseData, new Callback() {
            @Override
            public void onFailure(Call call, final IOException e) {
                progress.dismiss();
                Log.e(TAG, "Failed supported versions by DS request", e);

                activity.showDialog("Failed to send the request: " + e.getMessage(), "OK");
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                progress.dismiss();
                String responseBody = response.body().string();
                Log.d(TAG, "Supported versions by DS response: " + responseBody);
                if (response.isSuccessful()) {
                    SupportedVersionsResponse parsedResponse = new Gson().fromJson(responseBody, SupportedVersionsResponse.class);
                    List<DirectoryServerItem> directoryServers = parsedResponse.getDirectoryServers();
                    if (!directoryServers.isEmpty()) {
                        DirectoryServerItem ds = directoryServers.get(0);
                        purchaseWithDeterminedDs(purchaseData, ds, parsedResponse.getExtraAuthParams(), directoryServers.size() > 1);
                    } else {
                        activity.showDialog("There is no DS to continue", "OK");
                    }
                } else {
                    activity.showDialog("Invalid supported versions by DS response: " + responseBody, "OK");
                }
            }
        });
    }

    private void purchaseWithDeterminedDs(final PurchaseData purchaseData, final DirectoryServerItem directoryServerItem,
                                          final Map<String, Object> extraAuthParams, final boolean dualBrandedCard) {
        CreateTransactionSpec createTransactionSpec = new CreateTransactionSpec();
        createTransactionSpec.setCardId(purchaseData.getAccountNumber());
        createTransactionSpec.setDirectoryServerId(directoryServerItem.getDsIdentifier());
        createTransactionSpec.setMessageVersion(directoryServerItem.getMessageVersion());
        final Transaction transaction = threeDS2Service.createTransaction(createTransactionSpec);

        if (transaction.isPromptRequested()) {
            transaction.doPrompt(activity, new UsdkPromptCallback() {
                @Override
                public void onCompleted() {
                    continuePurchaseWithDeterminedDs(transaction, purchaseData, directoryServerItem, extraAuthParams, dualBrandedCard);
                }

                @Override
                public void onCancel() {

                }
            });
        } else {
            continuePurchaseWithDeterminedDs(transaction, purchaseData, directoryServerItem, extraAuthParams, dualBrandedCard);
        }
    }

    private void continuePurchaseWithDeterminedDs(final Transaction transaction, final PurchaseData purchaseData,
                                                  DirectoryServerItem directoryServerItem, Map<String, Object> extraAuthParams,
                                                  boolean dualBrandedCard) {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                progress = transaction.getProgressView(activity);
                progress.show();
            }
        });

        AuthenticationRequestParameters authParams = transaction.getAuthenticationRequestParameters();
        PurchaseRequest authenticationRequest = new PurchaseRequest(authParams, purchaseData.getAccountNumber());
        Map<String, Object> purchaseRequest = generateAuthenticationRequest(purchaseData, authParams, directoryServerItem, extraAuthParams, dualBrandedCard);

        Log.d(TAG, "Auth request: " + authenticationRequest.toJson());
        final Request request = new Request.Builder()
                .url(purchaseData.getBackendUrl() + AUTH_PATH)
                .header("Authorization", String.format("Bearer %s:%s", purchaseData.getEmail(), purchaseData.getPassword()))
                .post(RequestBody.create(JSON, new Gson().toJson(purchaseRequest)))
                .build();

        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(final Call call, final IOException e) {
                progress.dismiss();
                Log.e(TAG, "Failed purchase request", e);

                activity.showDialog("Failed to send the request: " + e.getMessage(), "OK");

                transaction.close();
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                progress.dismiss();

                String responseBody = response.body().string();
                Log.d(TAG, "Purchase response: " + responseBody);

                final PurchaseResponse purchaseResponse = new Gson().fromJson(responseBody, PurchaseResponse.class);
                lastTransactionId = purchaseResponse.getThreeDSServerTransID();
                Log.d(TAG, "threeDSServerTransID: " + lastTransactionId);

                if (purchaseResponse.getTransStatus() != null
                        && purchaseResponse.getTransStatus().equals(CHALLENGE_TRANS_STATUS)) {

                    ChallengeParameters challengeParameters = new ChallengeParameters();
                    challengeParameters.setAcsRefNumber(purchaseResponse.getAcsReferenceNumber());
                    challengeParameters.set3DSServerTransactionID(purchaseResponse.getThreeDSServerTransID());
                    challengeParameters.setAcsSignedContent(purchaseResponse.getAcsSignedContent());
                    challengeParameters.setAcsTransactionID(purchaseResponse.getAcsTransID());

                    transaction.doChallenge(activity, challengeParameters, new ChallengeStatusReceiver() {
                        @Override
                        public void completed(final CompletionEvent completionEvent) {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            String message = String.format("The completed callback called, threeDSServerTransID=%s, status=%s", lastTransactionId, completionEvent.getTransactionStatus());
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void cancelled() {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            String message = String.format("The cancelled callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void timedout() {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            String message = String.format("The timedout callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void protocolError(ProtocolErrorEvent protocolErrorEvent) {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            Log.d(TAG, String.format("The runtimeError callback called, error message: %s", protocolErrorEvent.getErrorMessage()));
                            String message = String.format("The protocolError callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }

                        @Override
                        public void runtimeError(RuntimeErrorEvent runtimeErrorEvent) {
                            if (lastTransactionId != null) {
                                Log.d(TAG, "Transaction ID: " + lastTransactionId);
                            }

                            Log.d(TAG, String.format("The runtimeError callback called, error code: %s, error message: %s", runtimeErrorEvent.getErrorCode(), runtimeErrorEvent.getErrorMessage()));
                            String message = String.format("The runtimeError callback called, threeDSServerTransID=%s", lastTransactionId);
                            activity.showDialog(message, "OK");
                            transaction.close();
                        }
                    }, 30);
                } else {
                    closeTransactionWithMessageExtensions(transaction, purchaseResponse.getMessageExtension());

                    if (lastTransactionId != null) {
                        Log.d(TAG, "Transaction ID: " + lastTransactionId);
                    }
                    String message = String.format("The purchase completed, threeDSServerTransID=%s, status=%s", lastTransactionId, purchaseResponse.getTransStatus());
                    activity.showDialog(message, "OK");
                }
            }
        });
    }

    private void getSupportedVersionsByDsRequest(PurchaseData purchaseData, Callback callback) {
        Map<String, Object> supportedVersionsRequest = new HashMap<>();
        supportedVersionsRequest.put("cardID", purchaseData.getAccountNumber());
        supportedVersionsRequest.put("sdkProtocolVersions", new ArrayList<String>(){{
            add("2.1.0");
            add("2.2.0");
        }});

        Request request = new Request.Builder()
                .url(purchaseData.getBackendUrl() + SUPPORTED_VERSIONS_PATH)
                .header("Authorization", String.format("Bearer %s:%s", purchaseData.getEmail(), purchaseData.getPassword()))
                .post(RequestBody.create(JSON, new Gson().toJson(supportedVersionsRequest)))
                .build();

        client.newCall(request).enqueue(callback);
    }

    private void closeTransactionWithMessageExtensions(Transaction transaction, List<MessageExtension> messageExtensions) {
        CloseTransactionSpec closeTransactionSpec = new CloseTransactionSpec();
        closeTransactionSpec.setMessageExtensions(messageExtensions);
        transaction.close(closeTransactionSpec);
    }
}
